@@ -1,5 +1,5 @@
 
-from .align import BaseScorer, EmbeddingScorer, ConstantScorer, create_embedding_scorer
+from .align import BaseScorer, EmbeddingScorer, ConstantScorer, LookupScorer
 from .align import get_horizontal_alignment, local_alignment
 from .align import get_alignment_ranges, get_alignment_string
 from .parallel_align import align_collections
