@@ -19,20 +19,45 @@ logger = logging.getLogger(__name__)
 
 class Match:
     def __init__(self, doc1, doc2, sim, 
-                 with_context=False, coll1=None, coll2=None, n_words=25):
+                 with_context=False, with_alignment=False,
+                 coll1=None, coll2=None, n_words=25):
+
+        self.with_context = with_context
         if with_context:
-            self._repr = Match.get_match_with_context(
+            self._meta = Match.get_match_with_context(
                 doc1, doc2, sim, coll1, coll2, n_words=n_words)
         else:
-            self._repr = Match.get_match(doc1, doc2, sim)
+            self._meta = Match.get_match(doc1, doc2, sim)
+
+        # add horizontal alignment
+        try:
+            a1, a2, _ = doc1.local_alignment(doc2, field='lemma')
+        except:
+            a1, a2, _ = doc1.local_alignment(doc2, field='token')
+        self._horizontal_alignment = doc1.get_horizontal_alignment(
+            doc2, a1=a1, a2=a2, field='token')
 
     def __repr__(self):
-        return self._repr
+        return self.get_printable_match()
+
+    def get_data(self):
+        return self._meta
+
+    def print_alignment(self):
+        print('Similarity -> {:.5f}\n\t{}\n\t{}\n\t{}'.format(
+                self._meta['similarity'], 
+                self._meta['doc1id'], 
+                '\t{}\n\t\t{}\n\t\t{}'.format(*self._horizontal_alignment),
+                self._meta['doc2id']))
 
     @staticmethod
     def get_match(doc1, doc2, sim):
-        return 'Similarity -> {:.5f}\n\t{}: {}\n\t{}: {}'.format(
-            sim, doc1.doc_id, doc1.text, doc2.doc_id, doc2.text)
+        return {
+            'similarity': sim, 
+            'doc1id': doc1.doc_id, 
+            'doc1text': doc1.text,
+            'doc2id': doc2.doc_id, 
+            'doc2text': doc2.text}
 
     @staticmethod
     def get_match_with_context(doc1, doc2, sim, coll1, coll2, n_words=25):
@@ -40,12 +65,34 @@ class Match:
         doc1right = coll1.get_doc_context_right(coll1.get_doc_idx(doc1.doc_id), n_words)
         doc2left = coll2.get_doc_context_left(coll2.get_doc_idx(doc2.doc_id), n_words)
         doc2right = coll2.get_doc_context_right(coll2.get_doc_idx(doc2.doc_id), n_words)
-        return 'Similarity -> {:.5f}\n\t{}:\n\t{}\n\n\t{}:\n\t{}'.format(
-            sim,
-            doc1.doc_id, '{}\n\t    {}\n\t{}'.format(
-                ' '.join(doc1left), doc1.text, ' '.join(doc1right)),
-            doc2.doc_id, '{}\n\t    {}\n\t{}'.format(
-                ' '.join(doc2left), doc2.text, ' '.join(doc2right)))
+        return {
+            'similarity': sim, 
+            'doc1id': doc1.doc_id,
+            'doc1left': ' '.join(doc1left),
+            'doc1text': doc1.text,
+            'doc1right': ' '.join(doc1right),
+            'doc2id': doc2.doc_id, 
+            'doc2left': ' '.join(doc2left),
+            'doc2text': doc2.text,
+            'doc2right': ' '.join(doc2right)}
+
+    def get_printable_match(self):
+        if self.with_context:
+            return 'Similarity -> {:.5f}\n\t{}:\n\t{}\n\n\t{}:\n\t{}'.format(
+                self._meta['similarity'],
+                self._meta['doc1id'],
+                '{}\n\t    {}\n\t{}'.format(
+                    self._meta['doc1left'], self._meta['doc1text'], self._meta['doc1right']),
+                self._meta['doc2id'],
+                '{}\n\t    {}\n\t{}'.format(
+                    self._meta['doc2left'], self._meta['doc2text'], self._meta['doc2right']))
+        else:
+            return 'Similarity -> {:.5f}\n\t{}: {}\n\t{}: {}'.format(
+                self._meta['similarity'], 
+                self._meta['doc1id'], 
+                self._meta['doc1text'],
+                self._meta['doc2id'],
+                self._meta['doc2text'])
 
 
 class Results:
@@ -70,14 +117,32 @@ class Results:
         sparse_utils.set_threshold(self.sims, min_sim)
 
     def get_top_matches(self, n=None, min_sim=0, max_sim=None, sample=False, 
-                        with_context=False, n_words=25):
+                        with_context=False, n_words=25, 
+                        whitelist_x=None, whitelist_y=None,
+                        blacklist_x=None, blacklist_y=None):
+
         x, y, _ = sparse.find(self.sims > min_sim)
         score = self.sims[x, y]
         # sparse.find returns a scipy matrix instead of a np array
         score = np.array(score)[0]
         if max_sim is not None:
-            index, = np.where(score <= max_sim)
-            x, y, score = x[index], y[index], score[index]
+            keep, = np.where(score <= max_sim)
+            x, y, score = x[keep], y[keep], score[keep]
+
+        # white/blacklisting
+        if whitelist_x is not None:
+            drop, = np.where(~np.isin(x, whitelist_x))
+            x, y, score = np.delete(x, drop), np.delete(y, drop), np.delete(score, drop)
+        elif blacklist_x is not None:
+            drop, = np.where(np.isin(x, blacklist_x))
+            x, y, score = np.delete(x, drop), np.delete(y, drop), np.delete(score, drop)
+        if whitelist_y is not None:
+            drop, = np.where(~np.isin(y, whitelist_y))
+            x, y, score = np.delete(x, drop), np.delete(y, drop), np.delete(score, drop)
+        elif blacklist_y is not None:
+            drop, = np.where(np.isin(y, blacklist_y))
+            x, y, score = np.delete(x, drop), np.delete(y, drop), np.delete(score, drop)
+
         index = np.argsort(score)[::-1]
         if sample and n is not None:
             index = np.random.choice(np.arange(len(index)), size=n, replace=False)
@@ -89,6 +154,24 @@ class Results:
                 with_context=with_context, 
                 coll1=self.coll1, coll2=self.coll2, n_words=n_words)
 
+    def export_top_matches_to_csv(self, outputpath, tuple_sep='+++', **kwargs):
+        with open(outputpath, 'w+') as f:
+            header = None
+            for m in self.get_top_matches(**kwargs):
+                if header is None:
+                    header = list(m.get_data())
+                    f.write('\t'.join(header) + '\n')
+                row = ''
+                for key, val in m.get_data().items():
+                    if isinstance(val, float):
+                        val = '{:.5f}'.format(val)
+                    elif isinstance(val, tuple):
+                        val = tuple_sep.join(map(str, val))
+                    else:
+                        val = str(val)
+                    row += ('\t' if row else '') + val
+                f.write(row + '\n')
+
 
 def pipeline(coll1, coll2=None,
              # Text Preprocessing
